@@ -22,6 +22,8 @@
 
 """Generates tarballs for Git trees."""
 
+__all__ = ["ChunkedBytesIO", "tar_stream"]
+
 import posixpath
 import stat
 import struct
@@ -30,7 +32,7 @@ from collections.abc import Generator
 from contextlib import closing
 from io import BytesIO
 from os import SEEK_END
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from .object_store import BaseObjectStore
@@ -59,7 +61,7 @@ class ChunkedBytesIO:
         self.contents = contents
         self.pos = (0, 0)
 
-    def read(self, maxbytes: Optional[int] = None) -> bytes:
+    def read(self, maxbytes: int | None = None) -> bytes:
         """Read bytes from the chunked stream.
 
         Args:
@@ -135,6 +137,7 @@ def tar_stream(
             buf.seek(0, SEEK_END)
 
         for entry_abspath, entry in _walk_tree(store, tree, prefix):
+            assert entry.sha is not None
             try:
                 blob = store[entry.sha]
             except KeyError:
@@ -151,6 +154,7 @@ def tar_stream(
             # tarfile only works with ascii.
             info.name = entry_abspath.decode("utf-8", "surrogateescape")
             info.size = blob.raw_length()
+            assert entry.mode is not None
             info.mode = entry.mode
             info.mtime = mtime
 
@@ -166,8 +170,11 @@ def _walk_tree(
 ) -> Generator[tuple[bytes, "TreeEntry"], None, None]:
     """Recursively walk a dulwich Tree, yielding tuples of (absolute path, TreeEntry) along the way."""
     for entry in tree.iteritems():
+        assert entry.path is not None
         entry_abspath = posixpath.join(root, entry.path)
+        assert entry.mode is not None
         if stat.S_ISDIR(entry.mode):
+            assert entry.sha is not None
             subtree = store[entry.sha]
             if isinstance(subtree, Tree):
                 yield from _walk_tree(store, subtree, entry_abspath)
