@@ -22,9 +22,15 @@
 
 """Fast export/import functionality."""
 
+__all__ = [
+    "GitFastExporter",
+    "GitImportProcessor",
+    "split_email",
+]
+
 import stat
 from collections.abc import Generator
-from typing import TYPE_CHECKING, Any, BinaryIO, Optional
+from typing import TYPE_CHECKING, Any, BinaryIO
 
 from fastimport import commands, parser, processor
 from fastimport import errors as fastimport_errors
@@ -66,7 +72,7 @@ class GitFastExporter:
         """
         self.outf = outf
         self.store = store
-        self.markers: dict[bytes, bytes] = {}
+        self.markers: dict[bytes, ObjectID] = {}
         self._marker_idx = 0
 
     def print_cmd(self, cmd: object) -> None:
@@ -101,7 +107,7 @@ class GitFastExporter:
         """
         marker = self._allocate_marker()
         self.markers[marker] = blob.id
-        return (commands.BlobCommand(marker, blob.data), marker)
+        return (commands.BlobCommand(marker, blob.data), marker)  # type: ignore[no-untyped-call,unused-ignore]
 
     def emit_blob(self, blob: Blob) -> bytes:
         """Emit a blob to the output stream.
@@ -117,7 +123,7 @@ class GitFastExporter:
         return marker
 
     def _iter_files(
-        self, base_tree: Optional[bytes], new_tree: Optional[bytes]
+        self, base_tree: ObjectID | None, new_tree: ObjectID | None
     ) -> Generator[Any, None, None]:
         for (
             (old_path, new_path),
@@ -126,7 +132,7 @@ class GitFastExporter:
         ) in self.store.tree_changes(base_tree, new_tree):
             if new_path is None:
                 if old_path is not None:
-                    yield commands.FileDeleteCommand(old_path)
+                    yield commands.FileDeleteCommand(old_path)  # type: ignore[no-untyped-call,unused-ignore]
                 continue
             marker = b""
             if new_mode is not None and not stat.S_ISDIR(new_mode):
@@ -137,15 +143,16 @@ class GitFastExporter:
                     if isinstance(blob, Blob):
                         marker = self.emit_blob(blob)
             if old_path != new_path and old_path is not None:
-                yield commands.FileRenameCommand(old_path, new_path)
+                yield commands.FileRenameCommand(old_path, new_path)  # type: ignore[no-untyped-call,unused-ignore]
             if old_mode != new_mode or old_hexsha != new_hexsha:
                 prefixed_marker = b":" + marker
-                yield commands.FileModifyCommand(
+                assert new_mode is not None
+                yield commands.FileModifyCommand(  # type: ignore[no-untyped-call,unused-ignore]
                     new_path, new_mode, prefixed_marker, None
                 )
 
     def _export_commit(
-        self, commit: Commit, ref: Ref, base_tree: Optional[ObjectID] = None
+        self, commit: Commit, ref: Ref, base_tree: ObjectID | None = None
     ) -> tuple[Any, bytes]:
         file_cmds = list(self._iter_files(base_tree, commit.tree))
         marker = self._allocate_marker()
@@ -157,7 +164,7 @@ class GitFastExporter:
             merges = []
         author, author_email = split_email(commit.author)
         committer, committer_email = split_email(commit.committer)
-        cmd = commands.CommitCommand(
+        cmd = commands.CommitCommand(  # type: ignore[no-untyped-call,unused-ignore]
             ref,
             marker,
             (author, author_email, commit.author_time, commit.author_timezone),
@@ -175,7 +182,7 @@ class GitFastExporter:
         return (cmd, marker)
 
     def emit_commit(
-        self, commit: Commit, ref: Ref, base_tree: Optional[ObjectID] = None
+        self, commit: Commit, ref: Ref, base_tree: ObjectID | None = None
     ) -> bytes:
         """Emit a commit in fast-export format.
 
@@ -192,7 +199,7 @@ class GitFastExporter:
         return marker
 
 
-class GitImportProcessor(processor.ImportProcessor):
+class GitImportProcessor(processor.ImportProcessor):  # type: ignore[misc,unused-ignore]
     """An import processor that imports into a Git repository using Dulwich."""
 
     # FIXME: Batch creation of objects?
@@ -200,9 +207,9 @@ class GitImportProcessor(processor.ImportProcessor):
     def __init__(
         self,
         repo: "BaseRepo",
-        params: Optional[Any] = None,  # noqa: ANN401
+        params: Any | None = None,  # noqa: ANN401
         verbose: bool = False,
-        outf: Optional[BinaryIO] = None,
+        outf: BinaryIO | None = None,
     ) -> None:
         """Initialize GitImportProcessor.
 
@@ -212,10 +219,10 @@ class GitImportProcessor(processor.ImportProcessor):
           verbose: Whether to enable verbose output
           outf: Output file for verbose messages
         """
-        processor.ImportProcessor.__init__(self, params, verbose)
+        processor.ImportProcessor.__init__(self, params, verbose)  # type: ignore[no-untyped-call,unused-ignore]
         self.repo = repo
         self.last_commit = ZERO_SHA
-        self.markers: dict[bytes, bytes] = {}
+        self.markers: dict[bytes, ObjectID] = {}
         self._contents: dict[bytes, tuple[int, bytes]] = {}
 
     def lookup_object(self, objectish: bytes) -> ObjectID:
@@ -229,9 +236,9 @@ class GitImportProcessor(processor.ImportProcessor):
         """
         if objectish.startswith(b":"):
             return self.markers[objectish[1:]]
-        return objectish
+        return ObjectID(objectish)
 
-    def import_stream(self, stream: BinaryIO) -> dict[bytes, bytes]:
+    def import_stream(self, stream: BinaryIO) -> dict[bytes, ObjectID]:
         """Import from a fast-import stream.
 
         Args:
@@ -240,8 +247,8 @@ class GitImportProcessor(processor.ImportProcessor):
         Returns:
           Dictionary of markers to object IDs
         """
-        p = parser.ImportParser(stream)
-        self.process(p.iter_commands)
+        p = parser.ImportParser(stream)  # type: ignore[no-untyped-call,unused-ignore]
+        self.process(p.iter_commands)  # type: ignore[no-untyped-call,unused-ignore]
         return self.markers
 
     def blob_handler(self, cmd: commands.BlobCommand) -> None:
@@ -258,19 +265,28 @@ class GitImportProcessor(processor.ImportProcessor):
         """Process a CommitCommand."""
         commit = Commit()
         if cmd.author is not None:
-            author = cmd.author
+            (author_name, author_email, author_timestamp, author_timezone) = cmd.author
         else:
-            author = cmd.committer
-        (author_name, author_email, author_timestamp, author_timezone) = author
+            (author_name, author_email, author_timestamp, author_timezone) = (
+                cmd.committer
+            )
         (
             committer_name,
             committer_email,
             commit_timestamp,
             commit_timezone,
         ) = cmd.committer
+        if isinstance(author_name, str):
+            author_name = author_name.encode("utf-8")
+        if isinstance(author_email, str):
+            author_email = author_email.encode("utf-8")
         commit.author = author_name + b" <" + author_email + b">"
         commit.author_timezone = author_timezone
         commit.author_time = int(author_timestamp)
+        if isinstance(committer_name, str):
+            committer_name = committer_name.encode("utf-8")
+        if isinstance(committer_email, str):
+            committer_email = committer_email.encode("utf-8")
         commit.committer = committer_name + b" <" + committer_email + b">"
         commit.commit_timezone = commit_timezone
         commit.commit_time = int(commit_timestamp)
@@ -279,29 +295,39 @@ class GitImportProcessor(processor.ImportProcessor):
         if cmd.from_:
             cmd.from_ = self.lookup_object(cmd.from_)
             self._reset_base(cmd.from_)
-        for filecmd in cmd.iter_files():
+        for filecmd in cmd.iter_files():  # type: ignore[no-untyped-call,unused-ignore]
             if filecmd.name == b"filemodify":
+                assert isinstance(filecmd, commands.FileModifyCommand)
                 if filecmd.data is not None:
                     blob = Blob.from_string(filecmd.data)
                     self.repo.object_store.add_object(blob)
                     blob_id = blob.id
                 else:
+                    assert filecmd.dataref is not None
                     blob_id = self.lookup_object(filecmd.dataref)
                 self._contents[filecmd.path] = (filecmd.mode, blob_id)
             elif filecmd.name == b"filedelete":
+                assert isinstance(filecmd, commands.FileDeleteCommand)
                 del self._contents[filecmd.path]
             elif filecmd.name == b"filecopy":
+                assert isinstance(filecmd, commands.FileCopyCommand)
                 self._contents[filecmd.dest_path] = self._contents[filecmd.src_path]
             elif filecmd.name == b"filerename":
+                assert isinstance(filecmd, commands.FileRenameCommand)
                 self._contents[filecmd.new_path] = self._contents[filecmd.old_path]
                 del self._contents[filecmd.old_path]
             elif filecmd.name == b"filedeleteall":
                 self._contents = {}
             else:
-                raise Exception(f"Command {filecmd.name} not supported")
+                raise Exception(f"Command {filecmd.name!r} not supported")
+        from dulwich.objects import ObjectID
+
         commit.tree = commit_tree(
             self.repo.object_store,
-            ((path, hexsha, mode) for (path, (mode, hexsha)) in self._contents.items()),
+            (
+                (path, ObjectID(hexsha), mode)
+                for (path, (mode, hexsha)) in self._contents.items()
+            ),
         )
         if self.last_commit != ZERO_SHA:
             commit.parents.append(self.last_commit)
@@ -311,7 +337,12 @@ class GitImportProcessor(processor.ImportProcessor):
         self.repo[cmd.ref] = commit.id
         self.last_commit = commit.id
         if cmd.mark:
-            self.markers[cmd.mark] = commit.id
+            mark_bytes = (
+                cmd.mark
+                if isinstance(cmd.mark, bytes)
+                else str(cmd.mark).encode("ascii")
+            )
+            self.markers[mark_bytes] = commit.id
 
     def progress_handler(self, cmd: commands.ProgressCommand) -> None:
         """Process a ProgressCommand."""
@@ -333,16 +364,18 @@ class GitImportProcessor(processor.ImportProcessor):
                 mode,
                 hexsha,
             ) in iter_tree_contents(self.repo.object_store, tree_id):
+                assert path is not None and mode is not None and hexsha is not None
                 self._contents[path] = (mode, hexsha)
 
     def reset_handler(self, cmd: commands.ResetCommand) -> None:
         """Process a ResetCommand."""
+        from_: ObjectID
         if cmd.from_ is None:
             from_ = ZERO_SHA
         else:
             from_ = self.lookup_object(cmd.from_)
         self._reset_base(from_)
-        self.repo.refs[cmd.ref] = from_
+        self.repo.refs[Ref(cmd.ref)] = from_
 
     def tag_handler(self, cmd: commands.TagCommand) -> None:
         """Process a TagCommand."""
@@ -355,4 +388,9 @@ class GitImportProcessor(processor.ImportProcessor):
 
     def feature_handler(self, cmd: commands.FeatureCommand) -> None:
         """Process a FeatureCommand."""
-        raise fastimport_errors.UnknownFeature(cmd.feature_name)
+        feature_name = (
+            cmd.feature_name.decode("utf-8")
+            if isinstance(cmd.feature_name, bytes)
+            else cmd.feature_name
+        )
+        raise fastimport_errors.UnknownFeature(feature_name)  # type: ignore[no-untyped-call,unused-ignore]
