@@ -21,10 +21,23 @@
 
 """Implementation of Git filter drivers (clean/smudge filters)."""
 
+__all__ = [
+    "CompositeFilterDriver",
+    "FilterBlobNormalizer",
+    "FilterContext",
+    "FilterDriver",
+    "FilterError",
+    "FilterRegistry",
+    "ProcessFilterDriver",
+    "get_filter_for_path",
+]
+
 import logging
+import shlex
 import subprocess
 import threading
-from typing import TYPE_CHECKING, Callable, Optional
+from collections.abc import Callable
+from typing import TYPE_CHECKING
 from typing import Protocol as TypingProtocol
 
 from .attrs import GitAttributes
@@ -114,11 +127,11 @@ class ProcessFilterDriver:
 
     def __init__(
         self,
-        clean_cmd: Optional[str] = None,
-        smudge_cmd: Optional[str] = None,
+        clean_cmd: str | None = None,
+        smudge_cmd: str | None = None,
         required: bool = False,
-        cwd: Optional[str] = None,
-        process_cmd: Optional[str] = None,
+        cwd: str | None = None,
+        process_cmd: str | None = None,
     ) -> None:
         """Initialize ProcessFilterDriver.
 
@@ -134,15 +147,15 @@ class ProcessFilterDriver:
         self.required = required
         self.cwd = cwd
         self.process_cmd = process_cmd
-        self._process: Optional[subprocess.Popen[bytes]] = None
-        self._protocol: Optional[Protocol] = None
+        self._process: subprocess.Popen[bytes] | None = None
+        self._protocol: Protocol | None = None
         self._capabilities: set[bytes] = set()
         self._process_lock = threading.Lock()
 
-    def _get_or_start_process(self) -> Optional["Protocol"]:
+    def _get_or_start_process(self) -> "Protocol | None":
         """Get or start the long-running process filter."""
         if self._process is None and self.process_cmd:
-            from .errors import HangupException
+            from .errors import GitProtocolError, HangupException
             from .protocol import Protocol
 
             try:
@@ -216,7 +229,12 @@ class ProcessFilterDriver:
                     if cap.startswith(b"capability="):
                         self._capabilities.add(cap[11:])  # Remove "capability=" prefix
 
-            except (OSError, subprocess.SubprocessError, HangupException) as e:
+            except (
+                OSError,
+                subprocess.SubprocessError,
+                HangupException,
+                GitProtocolError,
+            ) as e:
                 self.cleanup()
                 raise FilterError(f"Failed to start process filter: {e}")
         return self._protocol
@@ -255,13 +273,16 @@ class ProcessFilterDriver:
                     pkt = self._protocol.read_pkt_line()
                     if pkt is None:  # flush packet ends headers
                         break
-                    key, _, value = pkt.decode().rstrip("\n\r").partition("=")
+                    key, _, value = pkt.rstrip(b"\n\r").partition(b"=")
                     response_headers[key] = value
 
                 # Check status
-                status = response_headers.get("status", "error")
-                if status != "success":
-                    raise FilterError(f"Process filter {operation} failed: {status}")
+                status = response_headers.get(b"status", b"error")
+                if status != b"success":
+                    status_str = status.decode("utf-8", errors="replace")
+                    raise FilterError(
+                        f"Process filter {operation} failed: {status_str}"
+                    )
 
                 # Read result data
                 result_chunks = []
@@ -278,14 +299,15 @@ class ProcessFilterDriver:
                     pkt = self._protocol.read_pkt_line()
                     if pkt is None:  # flush packet ends final headers
                         break
-                    key, _, value = pkt.decode().rstrip("\n\r").partition("=")
+                    key, _, value = pkt.rstrip(b"\n\r").partition(b"=")
                     final_headers[key] = value
 
                 # Check final status (if provided, it overrides the initial status)
-                final_status = final_headers.get("status", status)
-                if final_status != "success":
+                final_status = final_headers.get(b"status", status)
+                if final_status != b"success":
+                    final_status_str = final_status.decode("utf-8", errors="replace")
                     raise FilterError(
-                        f"Process filter {operation} failed with final status: {final_status}"
+                        f"Process filter {operation} failed with final status: {final_status_str}"
                     )
 
                 return b"".join(result_chunks)
@@ -297,6 +319,8 @@ class ProcessFilterDriver:
 
     def clean(self, data: bytes) -> bytes:
         """Apply clean filter using external process."""
+        import os
+
         # Try process filter first (much faster)
         if self.process_cmd:
             try:
@@ -304,7 +328,7 @@ class ProcessFilterDriver:
             except FilterError as e:
                 if self.required:
                     raise
-                logging.warning(f"Process filter failed, falling back: {e}")
+                logging.warning("Process filter failed, falling back: %s", e)
 
         # Fall back to clean command
         if not self.clean_cmd:
@@ -312,10 +336,15 @@ class ProcessFilterDriver:
                 raise FilterError("Clean command is required but not configured")
             return data
 
+        # Parse command into list of arguments
+        # Use shlex.split for proper handling of quoted arguments
+        # On Windows, shlex needs posix=False for correct parsing
+        cmd_args = shlex.split(self.clean_cmd, posix=(os.name != "nt"))
+
         try:
             result = subprocess.run(
-                self.clean_cmd,
-                shell=True,
+                cmd_args,
+                shell=False,
                 input=data,
                 capture_output=True,
                 check=True,
@@ -326,11 +355,13 @@ class ProcessFilterDriver:
             if self.required:
                 raise FilterError(f"Required clean filter failed: {e}")
             # If not required, log warning and return original data on failure
-            logging.warning(f"Optional clean filter failed: {e}")
+            logging.warning("Optional clean filter failed: %s", e)
             return data
 
     def smudge(self, data: bytes, path: bytes = b"") -> bytes:
         """Apply smudge filter using external process."""
+        import os
+
         path_str = path.decode("utf-8", errors="replace")
 
         # Try process filter first (much faster)
@@ -340,7 +371,7 @@ class ProcessFilterDriver:
             except FilterError as e:
                 if self.required:
                     raise
-                logging.warning(f"Process filter failed, falling back: {e}")
+                logging.warning("Process filter failed, falling back: %s", e)
 
         # Fall back to smudge command
         if not self.smudge_cmd:
@@ -348,13 +379,18 @@ class ProcessFilterDriver:
                 raise FilterError("Smudge command is required but not configured")
             return data
 
-        # Substitute %f placeholder with file path
-        cmd = self.smudge_cmd.replace("%f", path_str)
+        # Parse command into list of arguments and substitute %f placeholder
+        # Use shlex.split for proper handling of quoted arguments
+        # On Windows, shlex needs posix=False for correct parsing
+        cmd_args = shlex.split(self.smudge_cmd, posix=(os.name != "nt"))
+
+        # Replace %f placeholder with actual path
+        cmd_args = [arg.replace("%f", path_str) for arg in cmd_args]
 
         try:
             result = subprocess.run(
-                cmd,
-                shell=True,
+                cmd_args,
+                shell=False,
                 input=data,
                 capture_output=True,
                 check=True,
@@ -363,9 +399,11 @@ class ProcessFilterDriver:
             return result.stdout
         except subprocess.CalledProcessError as e:
             if self.required:
-                raise FilterError(f"Required smudge filter failed: {e}")
+                raise FilterError(
+                    f"Required smudge filter failed: {e} {e.stderr} {e.stdout}"
+                )
             # If not required, log warning and return original data on failure
-            logging.warning(f"Optional smudge filter failed: {e}")
+            logging.warning("Optional smudge filter failed: %s", e)
             return data
 
     def cleanup(self) -> None:
@@ -422,6 +460,22 @@ class ProcessFilterDriver:
                 except ProcessLookupError:
                     # Process already dead
                     pass
+
+            # Close stdout and stderr to prevent resource leaks
+            if self._process.stdout and not self._process.stdout.closed:
+                try:
+                    self._process.stdout.close()
+                except (OSError, ValueError):
+                    # OSError: I/O operation on closed file
+                    # ValueError: I/O operation on closed file (some platforms)
+                    pass
+
+            if self._process.stderr and not self._process.stderr.closed:
+                try:
+                    self._process.stderr.close()
+                except (OSError, ValueError):
+                    pass
+
         self._process = None
         self._protocol = None
 
@@ -503,7 +557,7 @@ class FilterContext:
         self.filter_registry = filter_registry
         self._active_drivers: dict[str, FilterDriver] = {}
 
-    def get_driver(self, name: str) -> Optional[FilterDriver]:
+    def get_driver(self, name: str) -> FilterDriver | None:
         """Get a filter driver by name, managing stateful instances.
 
         This method handles driver instantiation and caching. Only drivers
@@ -515,7 +569,7 @@ class FilterContext:
         Returns:
             FilterDriver instance or None
         """
-        driver: Optional[FilterDriver] = None
+        driver: FilterDriver | None = None
         # Check if we have a cached instance that should be reused
         if name in self._active_drivers:
             driver = self._active_drivers[name]
@@ -580,8 +634,8 @@ class FilterRegistry:
 
     def __init__(
         self,
-        config: Optional["StackedConfig"] = None,
-        repo: Optional["BaseRepo"] = None,
+        config: "StackedConfig | None" = None,
+        repo: "BaseRepo | None" = None,
     ) -> None:
         """Initialize FilterRegistry.
 
@@ -611,7 +665,7 @@ class FilterRegistry:
         """Register a filter driver instance."""
         self._drivers[name] = driver
 
-    def get_driver(self, name: str) -> Optional[FilterDriver]:
+    def get_driver(self, name: str) -> FilterDriver | None:
         """Get a filter driver by name."""
         # Check if we already have an instance
         if name in self._drivers:
@@ -646,14 +700,14 @@ class FilterRegistry:
             # Don't raise exceptions in __del__
             pass
 
-    def _create_from_config(self, name: str) -> Optional[FilterDriver]:
+    def _create_from_config(self, name: str) -> FilterDriver | None:
         """Create a filter driver from config."""
         if self.config is None:
             return None
 
-        clean_cmd: Optional[str] = None
-        smudge_cmd: Optional[str] = None
-        process_cmd: Optional[str] = None
+        clean_cmd: str | None = None
+        smudge_cmd: str | None = None
+        process_cmd: str | None = None
 
         # Get process command (preferred over clean/smudge for performance)
         try:
@@ -759,9 +813,9 @@ class FilterRegistry:
 def get_filter_for_path(
     path: bytes,
     gitattributes: "GitAttributes",
-    filter_registry: Optional[FilterRegistry] = None,
-    filter_context: Optional[FilterContext] = None,
-) -> Optional[FilterDriver]:
+    filter_registry: FilterRegistry | None = None,
+    filter_context: FilterContext | None = None,
+) -> FilterDriver | None:
     """Get the appropriate filter driver for a given path.
 
     Args:
@@ -857,11 +911,11 @@ class FilterBlobNormalizer:
 
     def __init__(
         self,
-        config_stack: Optional["StackedConfig"],
+        config_stack: "StackedConfig | None",
         gitattributes: GitAttributes,
-        filter_registry: Optional[FilterRegistry] = None,
-        repo: Optional["BaseRepo"] = None,
-        filter_context: Optional[FilterContext] = None,
+        filter_registry: FilterRegistry | None = None,
+        repo: "BaseRepo | None" = None,
+        filter_context: FilterContext | None = None,
     ) -> None:
         """Initialize FilterBlobNormalizer.
 
