@@ -36,10 +36,15 @@ module needs something else, it can import the standard logging module
 directly.
 """
 
+__all__ = [
+    "default_logging_config",
+    "getLogger",
+    "remove_null_handler",
+]
+
 import logging
 import os
 import sys
-from typing import Optional, Union
 
 getLogger = logging.getLogger
 
@@ -67,7 +72,7 @@ def _should_trace() -> bool:
     return True
 
 
-def _get_trace_target() -> Optional[Union[str, int]]:
+def _get_trace_target() -> str | int | None:
     """Get the trace target from GIT_TRACE environment variable.
 
     Returns:
