@@ -4,7 +4,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -22,8 +22,13 @@
 
 """A simple least-recently-used (LRU) cache."""
 
-from collections.abc import Iterable, Iterator
-from typing import Callable, Generic, Optional, TypeVar
+__all__ = [
+    "LRUCache",
+    "LRUSizeCache",
+]
+
+from collections.abc import Callable, Iterable, Iterator
+from typing import Generic, TypeVar, cast
 
 _null_key = object()
 
@@ -37,13 +42,15 @@ class _LRUNode(Generic[K, V]):
 
     __slots__ = ("cleanup", "key", "next_key", "prev", "size", "value")
 
-    prev: Optional["_LRUNode[K, V]"]
-    next_key: K
-    size: Optional[int]
+    prev: "_LRUNode[K, V] | None"
+    next_key: K | object
+    size: int | None
 
-    def __init__(self, key: K, value: V, cleanup=None) -> None:
+    def __init__(
+        self, key: K, value: V, cleanup: Callable[[K, V], None] | None = None
+    ) -> None:
         self.prev = None
-        self.next_key = _null_key  # type: ignore
+        self.next_key = _null_key
         self.key = key
         self.value = value
         self.cleanup = cleanup
@@ -70,12 +77,18 @@ class _LRUNode(Generic[K, V]):
 class LRUCache(Generic[K, V]):
     """A class which manages a cache of entries, removing unused ones."""
 
-    _least_recently_used: Optional[_LRUNode[K, V]]
-    _most_recently_used: Optional[_LRUNode[K, V]]
+    _least_recently_used: _LRUNode[K, V] | None
+    _most_recently_used: _LRUNode[K, V] | None
 
     def __init__(
-        self, max_cache: int = 100, after_cleanup_count: Optional[int] = None
+        self, max_cache: int = 100, after_cleanup_count: int | None = None
     ) -> None:
+        """Initialize LRUCache.
+
+        Args:
+          max_cache: Maximum number of entries to cache
+          after_cleanup_count: Number of entries to keep after cleanup
+        """
         self._cache: dict[K, _LRUNode[K, V]] = {}
         # The "HEAD" of the lru linked list
         self._most_recently_used = None
@@ -84,9 +97,11 @@ class LRUCache(Generic[K, V]):
         self._update_max_cache(max_cache, after_cleanup_count)
 
     def __contains__(self, key: K) -> bool:
+        """Check if key is in cache."""
         return key in self._cache
 
     def __getitem__(self, key: K) -> V:
+        """Get item from cache and mark as recently used."""
         cache = self._cache
         node = cache[key]
         # Inlined from _record_access to decrease the overhead of __getitem__
@@ -107,7 +122,7 @@ class LRUCache(Generic[K, V]):
             # 'next' item. So move the current lru to the previous node.
             self._least_recently_used = node_prev
         else:
-            node_next = cache[next_key]
+            node_next = cache[cast(K, next_key)]
             node_next.prev = node_prev
         assert node_prev
         assert mru
@@ -120,6 +135,7 @@ class LRUCache(Generic[K, V]):
         return node.value
 
     def __len__(self) -> int:
+        """Return number of items in cache."""
         return len(self._cache)
 
     def _walk_lru(self) -> Iterator[_LRUNode[K, V]]:
@@ -136,14 +152,14 @@ class LRUCache(Generic[K, V]):
             if node.next_key is _null_key:
                 if node is not self._least_recently_used:
                     raise AssertionError(
-                        "only the last node should have" f" no next value: {node}"
+                        f"only the last node should have no next value: {node}"
                     )
                 node_next = None
             else:
-                node_next = self._cache[node.next_key]
+                node_next = self._cache[cast(K, node.next_key)]
                 if node_next.prev is not node:
                     raise AssertionError(
-                        "inconsistency found, node.next.prev" f" != node: {node}"
+                        f"inconsistency found, node.next.prev != node: {node}"
                     )
             if node.prev is None:
                 if node is not self._most_recently_used:
@@ -154,13 +170,13 @@ class LRUCache(Generic[K, V]):
             else:
                 if node.prev.next_key != node.key:
                     raise AssertionError(
-                        "inconsistency found, node.prev.next" f" != node: {node}"
+                        f"inconsistency found, node.prev.next != node: {node}"
                     )
             yield node
             node = node_next
 
     def add(
-        self, key: K, value: V, cleanup: Optional[Callable[[K, V], None]] = None
+        self, key: K, value: V, cleanup: Callable[[K, V], None] | None = None
     ) -> None:
         """Add a new value to the cache.
 
@@ -193,7 +209,16 @@ class LRUCache(Generic[K, V]):
         """Get the number of entries we will cache."""
         return self._max_cache
 
-    def get(self, key: K, default: Optional[V] = None) -> Optional[V]:
+    def get(self, key: K, default: V | None = None) -> V | None:
+        """Get value from cache with default if not found.
+
+        Args:
+          key: Key to look up
+          default: Default value if key not found
+
+        Returns:
+          Value from cache or default
+        """
         node = self._cache.get(key, None)
         if node is None:
             return default
@@ -247,7 +272,7 @@ class LRUCache(Generic[K, V]):
         if node.prev is not None:
             node.prev.next_key = node.next_key
         if node.next_key is not _null_key:
-            node_next = self._cache[node.next_key]
+            node_next = self._cache[cast(K, node.next_key)]
             node_next.prev = node.prev
         # INSERT
         node.next_key = self._most_recently_used.key
@@ -267,11 +292,11 @@ class LRUCache(Generic[K, V]):
         if node.prev is not None:
             node.prev.next_key = node.next_key
         if node.next_key is not _null_key:
-            node_next = self._cache[node.next_key]
+            node_next = self._cache[cast(K, node.next_key)]
             node_next.prev = node.prev
         # And remove this node's pointers
         node.prev = None
-        node.next_key = _null_key  # type: ignore
+        node.next_key = _null_key
 
     def _remove_lru(self) -> None:
         """Remove one entry from the lru, and handle consequences.
@@ -288,11 +313,13 @@ class LRUCache(Generic[K, V]):
         while self._cache:
             self._remove_lru()
 
-    def resize(self, max_cache: int, after_cleanup_count: Optional[int] = None) -> None:
+    def resize(self, max_cache: int, after_cleanup_count: int | None = None) -> None:
         """Change the number of entries that will be cached."""
         self._update_max_cache(max_cache, after_cleanup_count=after_cleanup_count)
 
-    def _update_max_cache(self, max_cache, after_cleanup_count=None) -> None:
+    def _update_max_cache(
+        self, max_cache: int, after_cleanup_count: int | None = None
+    ) -> None:
         self._max_cache = max_cache
         if after_cleanup_count is None:
             self._after_cleanup_count = self._max_cache * 8 / 10
@@ -316,8 +343,8 @@ class LRUSizeCache(LRUCache[K, V]):
     def __init__(
         self,
         max_size: int = 1024 * 1024,
-        after_cleanup_size: Optional[int] = None,
-        compute_size: Optional[Callable[[V], int]] = None,
+        after_cleanup_size: int | None = None,
+        compute_size: Callable[[V], int] | None = None,
     ) -> None:
         """Create a new LRUSizeCache.
 
@@ -335,14 +362,14 @@ class LRUSizeCache(LRUCache[K, V]):
         """
         self._value_size = 0
         if compute_size is None:
-            self._compute_size = len  # type: ignore
+            self._compute_size = cast(Callable[[V], int], len)
         else:
             self._compute_size = compute_size
         self._update_max_size(max_size, after_cleanup_size=after_cleanup_size)
         LRUCache.__init__(self, max_cache=max(int(max_size / 512), 1))
 
     def add(
-        self, key: K, value: V, cleanup: Optional[Callable[[K, V], None]] = None
+        self, key: K, value: V, cleanup: Callable[[K, V], None] | None = None
     ) -> None:
         """Add a new value to the cache.
 
@@ -397,14 +424,14 @@ class LRUSizeCache(LRUCache[K, V]):
         self._value_size -= node.size
         LRUCache._remove_node(self, node)
 
-    def resize(self, max_size: int, after_cleanup_size: Optional[int] = None) -> None:
+    def resize(self, max_size: int, after_cleanup_size: int | None = None) -> None:
         """Change the number of bytes that will be cached."""
         self._update_max_size(max_size, after_cleanup_size=after_cleanup_size)
         max_cache = max(int(max_size / 512), 1)
         self._update_max_cache(max_cache)
 
     def _update_max_size(
-        self, max_size: int, after_cleanup_size: Optional[int] = None
+        self, max_size: int, after_cleanup_size: int | None = None
     ) -> None:
         self._max_size = max_size
         if after_cleanup_size is None:
