@@ -1,8 +1,9 @@
 # notes.py -- Git notes handling
 # Copyright (C) 2024 Jelmer Vernooij <jelmer@jelmer.uk>
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -20,20 +21,34 @@
 
 """Git notes handling."""
 
-import stat
-from collections.abc import Iterator
-from typing import TYPE_CHECKING, Optional
+__all__ = [
+    "DEFAULT_NOTES_REF",
+    "NOTES_REF_PREFIX",
+    "Notes",
+    "NotesTree",
+    "create_notes_tree",
+    "get_note_fanout_level",
+    "get_note_path",
+    "split_path_for_fanout",
+]
 
-from .objects import Blob, Tree
+import stat
+from collections.abc import Iterator, Sequence
+from typing import TYPE_CHECKING
+
+from .objects import Blob, ObjectID, Tree
+from .refs import Ref
 
 if TYPE_CHECKING:
     from .config import StackedConfig
+    from .object_store import BaseObjectStore
+    from .refs import RefsContainer
 
 NOTES_REF_PREFIX = b"refs/notes/"
 DEFAULT_NOTES_REF = NOTES_REF_PREFIX + b"commits"
 
 
-def get_note_fanout_level(tree: Tree, object_store) -> int:
+def get_note_fanout_level(tree: Tree, object_store: "BaseObjectStore") -> int:
     """Determine the fanout level for a note tree.
 
     Git uses a fanout directory structure for performance with large numbers
@@ -50,13 +65,25 @@ def get_note_fanout_level(tree: Tree, object_store) -> int:
 
     # Count the total number of notes in the tree recursively
     def count_notes(tree: Tree, level: int = 0) -> int:
+        """Count notes in a tree recursively.
+
+        Args:
+            tree: Tree to count notes in
+            level: Current recursion level
+
+        Returns:
+            Total number of notes
+        """
         count = 0
         for name, mode, sha in tree.items():
+            assert mode is not None
             if stat.S_ISREG(mode):
                 count += 1
             elif stat.S_ISDIR(mode) and level < 2:  # Only recurse 2 levels deep
+                assert sha is not None
                 try:
                     subtree = object_store[sha]
+                    assert isinstance(subtree, Tree)
                     count += count_notes(subtree, level + 1)
                 except KeyError:
                     pass
@@ -111,7 +138,7 @@ def get_note_path(object_sha: bytes, fanout_level: int = 0) -> bytes:
 class NotesTree:
     """Represents a Git notes tree."""
 
-    def __init__(self, tree: Tree, object_store):
+    def __init__(self, tree: Tree, object_store: "BaseObjectStore") -> None:
         """Initialize a notes tree.
 
         Args:
@@ -137,6 +164,8 @@ class NotesTree:
         dir_names = []
 
         for name, mode, sha in self._tree.items():
+            assert name is not None
+            assert mode is not None
             if stat.S_ISDIR(mode):
                 has_dirs = True
                 dir_names.append(name)
@@ -145,9 +174,12 @@ class NotesTree:
 
         # If we have files at the root level, check if they're full SHA names
         if has_files and not has_dirs:
-            # Check if any file names are full 40-char hex strings
+            # Check if any file names are full hex strings (40 for SHA-1, 64 for SHA-256)
+            hex_length = self._object_store.object_format.hex_length
             for name, mode, sha in self._tree.items():
-                if stat.S_ISREG(mode) and len(name) == 40:
+                assert name is not None
+                assert mode is not None
+                if stat.S_ISREG(mode) and len(name) == hex_length:
                     try:
                         int(name, 16)  # Verify it's a valid hex string
                         return 0  # No fanout
@@ -165,12 +197,15 @@ class NotesTree:
                 # Check a sample directory to determine if it's level 1 or 2
                 sample_dir_name = dir_names[0]
                 try:
-                    sample_mode, sample_sha = self._tree[sample_dir_name]
+                    _sample_mode, sample_sha = self._tree[sample_dir_name]
                     sample_tree = self._object_store[sample_sha]
+                    assert isinstance(sample_tree, Tree)
 
                     # Check if this subtree also has 2-char hex directories
                     sub_has_dirs = False
                     for sub_name, sub_mode, sub_sha in sample_tree.items():
+                        assert sub_name is not None
+                        assert sub_mode is not None
                         if stat.S_ISDIR(sub_mode) and len(sub_name) == 2:
                             try:
                                 int(sub_name, 16)
@@ -218,12 +253,27 @@ class NotesTree:
             components = path.split(b"/")
 
             # Build new tree structure
-            def update_tree(tree: Tree, components: list, blob_sha: bytes) -> Tree:
+            def update_tree(
+                tree: Tree, components: Sequence[bytes], blob_sha: ObjectID
+            ) -> Tree:
+                """Update tree with new note entry.
+
+                Args:
+                    tree: Tree to update
+                    components: Path components
+                    blob_sha: SHA of the note blob
+
+                Returns:
+                    Updated tree
+                """
                 if len(components) == 1:
                     # Leaf level - add the note blob
                     new_tree = Tree()
                     for name, mode, sha in tree.items():
                         if name != components[0]:
+                            assert name is not None
+                            assert mode is not None
+                            assert sha is not None
                             new_tree.add(name, mode, sha)
                     new_tree.add(components[0], stat.S_IFREG | 0o644, blob_sha)
                     return new_tree
@@ -234,8 +284,10 @@ class NotesTree:
                     for name, mode, sha in tree.items():
                         if name == components[0]:
                             # Update this subtree
+                            assert mode is not None and sha is not None
                             if stat.S_ISDIR(mode):
                                 subtree = self._object_store[sha]
+                                assert isinstance(subtree, Tree)
                             else:
                                 # If not a directory, we need to replace it
                                 subtree = Tree()
@@ -244,6 +296,11 @@ class NotesTree:
                             new_tree.add(name, stat.S_IFDIR, new_subtree.id)
                             found = True
                         else:
+                            assert (
+                                name is not None
+                                and mode is not None
+                                and sha is not None
+                            )
                             new_tree.add(name, mode, sha)
 
                     if not found:
@@ -259,7 +316,7 @@ class NotesTree:
             self._object_store.add_object(self._tree)
 
     def _update_tree_entry(
-        self, tree: Tree, name: bytes, mode: int, sha: bytes
+        self, tree: Tree, name: bytes, mode: int, sha: ObjectID
     ) -> Tree:
         """Update a tree entry and return the updated tree.
 
@@ -275,6 +332,11 @@ class NotesTree:
         new_tree = Tree()
         for existing_name, existing_mode, existing_sha in tree.items():
             if existing_name != name:
+                assert (
+                    existing_name is not None
+                    and existing_mode is not None
+                    and existing_sha is not None
+                )
                 new_tree.add(existing_name, existing_mode, existing_sha)
         new_tree.add(name, mode, sha)
         self._object_store.add_object(new_tree)
@@ -285,7 +347,7 @@ class NotesTree:
 
         return new_tree
 
-    def _get_note_sha(self, object_sha: bytes) -> Optional[bytes]:
+    def _get_note_sha(self, object_sha: bytes) -> ObjectID | None:
         """Get the SHA of the note blob for an object.
 
         Args:
@@ -303,7 +365,9 @@ class NotesTree:
                 mode, sha = current_tree[component]
                 if not stat.S_ISDIR(mode):  # Not a directory
                     return None
-                current_tree = self._object_store[sha]
+                obj = self._object_store[sha]
+                assert isinstance(obj, Tree)
+                current_tree = obj
             except KeyError:
                 return None
 
@@ -315,7 +379,7 @@ class NotesTree:
         except KeyError:
             return None
 
-    def get_note(self, object_sha: bytes) -> Optional[bytes]:
+    def get_note(self, object_sha: bytes) -> bytes | None:
         """Get the note content for an object.
 
         Args:
@@ -332,7 +396,8 @@ class NotesTree:
             note_obj = self._object_store[note_sha]
             if not isinstance(note_obj, Blob):
                 return None
-            return note_obj.data
+            data: bytes = note_obj.data
+            return data
         except KeyError:
             return None
 
@@ -360,12 +425,25 @@ class NotesTree:
         components = path.split(b"/")
 
         # Build new tree structure
-        def update_tree(tree: Tree, components: list, blob_sha: bytes) -> Tree:
+        def update_tree(
+            tree: Tree, components: Sequence[bytes], blob_sha: ObjectID
+        ) -> Tree:
+            """Update tree with new note entry.
+
+            Args:
+                tree: Tree to update
+                components: Path components
+                blob_sha: SHA of the note blob
+
+            Returns:
+                Updated tree
+            """
             if len(components) == 1:
                 # Leaf level - add the note blob
                 new_tree = Tree()
                 for name, mode, sha in tree.items():
                     if name != components[0]:
+                        assert name is not None and mode is not None and sha is not None
                         new_tree.add(name, mode, sha)
                 new_tree.add(components[0], stat.S_IFREG | 0o644, blob_sha)
                 return new_tree
@@ -376,8 +454,10 @@ class NotesTree:
                 for name, mode, sha in tree.items():
                     if name == components[0]:
                         # Update this subtree
+                        assert mode is not None and sha is not None
                         if stat.S_ISDIR(mode):
                             subtree = self._object_store[sha]
+                            assert isinstance(subtree, Tree)
                         else:
                             # If not a directory, we need to replace it
                             subtree = Tree()
@@ -386,6 +466,7 @@ class NotesTree:
                         new_tree.add(name, stat.S_IFDIR, new_subtree.id)
                         found = True
                     else:
+                        assert name is not None and mode is not None and sha is not None
                         new_tree.add(name, mode, sha)
 
                 if not found:
@@ -403,7 +484,7 @@ class NotesTree:
         self._fanout_level = self._detect_fanout_level()
         return new_tree
 
-    def remove_note(self, object_sha: bytes) -> Optional[Tree]:
+    def remove_note(self, object_sha: bytes) -> Tree | None:
         """Remove a note for an object.
 
         Args:
@@ -420,13 +501,23 @@ class NotesTree:
         components = path.split(b"/")
 
         # Build new tree structure without the note
-        def remove_from_tree(tree: Tree, components: list) -> Optional[Tree]:
+        def remove_from_tree(tree: Tree, components: Sequence[bytes]) -> Tree | None:
+            """Remove note entry from tree.
+
+            Args:
+                tree: Tree to remove from
+                components: Path components
+
+            Returns:
+                Updated tree or None if empty
+            """
             if len(components) == 1:
                 # Leaf level - remove the note
                 new_tree = Tree()
                 found = False
                 for name, mode, sha in tree.items():
                     if name != components[0]:
+                        assert name is not None and mode is not None and sha is not None
                         new_tree.add(name, mode, sha)
                     else:
                         found = True
@@ -441,9 +532,11 @@ class NotesTree:
                 new_tree = Tree()
                 modified = False
                 for name, mode, sha in tree.items():
+                    assert name is not None and mode is not None and sha is not None
                     if name == components[0] and stat.S_ISDIR(mode):
                         # Update this subtree
                         subtree = self._object_store[sha]
+                        assert isinstance(subtree, Tree)
                         new_subtree = remove_from_tree(subtree, components[1:])
                         if new_subtree is not None:
                             self._object_store.add_object(new_subtree)
@@ -467,27 +560,40 @@ class NotesTree:
         self._fanout_level = self._detect_fanout_level()
         return new_tree
 
-    def list_notes(self) -> Iterator[tuple[bytes, bytes]]:
+    def list_notes(self) -> Iterator[tuple[ObjectID, ObjectID]]:
         """List all notes in this tree.
 
         Yields:
             Tuples of (object_sha, note_sha)
         """
 
-        def walk_tree(tree: Tree, prefix: bytes = b"") -> Iterator[tuple[bytes, bytes]]:
+        def walk_tree(
+            tree: Tree, prefix: bytes = b""
+        ) -> Iterator[tuple[ObjectID, ObjectID]]:
+            """Walk the notes tree recursively.
+
+            Args:
+                tree: Tree to walk
+                prefix: Path prefix for current level
+
+            Yields:
+                Tuples of (object_sha, note_sha)
+            """
             for name, mode, sha in tree.items():
+                assert name is not None and mode is not None and sha is not None
                 if stat.S_ISDIR(mode):  # Directory
                     subtree = self._object_store[sha]
+                    assert isinstance(subtree, Tree)
                     yield from walk_tree(subtree, prefix + name)
                 elif stat.S_ISREG(mode):  # File
                     # Reconstruct the full hex SHA from the path
                     full_hex = prefix + name
-                    yield (full_hex, sha)
+                    yield (ObjectID(full_hex), sha)
 
         yield from walk_tree(self._tree)
 
 
-def create_notes_tree(object_store) -> Tree:
+def create_notes_tree(object_store: "BaseObjectStore") -> Tree:
     """Create an empty notes tree.
 
     Args:
@@ -504,7 +610,9 @@ def create_notes_tree(object_store) -> Tree:
 class Notes:
     """High-level interface for Git notes operations."""
 
-    def __init__(self, object_store, refs_container):
+    def __init__(
+        self, object_store: "BaseObjectStore", refs_container: "RefsContainer"
+    ) -> None:
         """Initialize Notes.
 
         Args:
@@ -516,9 +624,9 @@ class Notes:
 
     def get_notes_ref(
         self,
-        notes_ref: Optional[bytes] = None,
-        config: Optional["StackedConfig"] = None,
-    ) -> bytes:
+        notes_ref: bytes | None = None,
+        config: "StackedConfig | None" = None,
+    ) -> Ref:
         """Get the notes reference to use.
 
         Args:
@@ -533,14 +641,14 @@ class Notes:
                 notes_ref = config.get((b"notes",), b"displayRef")
             if notes_ref is None:
                 notes_ref = DEFAULT_NOTES_REF
-        return notes_ref
+        return Ref(notes_ref)
 
     def get_note(
         self,
         object_sha: bytes,
-        notes_ref: Optional[bytes] = None,
-        config: Optional["StackedConfig"] = None,
-    ) -> Optional[bytes]:
+        notes_ref: bytes | None = None,
+        config: "StackedConfig | None" = None,
+    ) -> bytes | None:
         """Get the note for an object.
 
         Args:
@@ -579,11 +687,11 @@ class Notes:
         self,
         object_sha: bytes,
         note_content: bytes,
-        notes_ref: Optional[bytes] = None,
-        author: Optional[bytes] = None,
-        committer: Optional[bytes] = None,
-        message: Optional[bytes] = None,
-        config: Optional["StackedConfig"] = None,
+        notes_ref: bytes | None = None,
+        author: bytes | None = None,
+        committer: bytes | None = None,
+        message: bytes | None = None,
+        config: "StackedConfig | None" = None,
     ) -> bytes:
         """Set or update a note for an object.
 
@@ -663,12 +771,12 @@ class Notes:
     def remove_note(
         self,
         object_sha: bytes,
-        notes_ref: Optional[bytes] = None,
-        author: Optional[bytes] = None,
-        committer: Optional[bytes] = None,
-        message: Optional[bytes] = None,
-        config: Optional["StackedConfig"] = None,
-    ) -> Optional[bytes]:
+        notes_ref: bytes | None = None,
+        author: bytes | None = None,
+        committer: bytes | None = None,
+        message: bytes | None = None,
+        config: "StackedConfig | None" = None,
+    ) -> bytes | None:
         """Remove a note for an object.
 
         Args:
@@ -744,9 +852,9 @@ class Notes:
 
     def list_notes(
         self,
-        notes_ref: Optional[bytes] = None,
-        config: Optional["StackedConfig"] = None,
-    ) -> list[tuple[bytes, bytes]]:
+        notes_ref: bytes | None = None,
+        config: "StackedConfig | None" = None,
+    ) -> list[tuple[ObjectID, bytes]]:
         """List all notes in a notes ref.
 
         Args:
@@ -778,7 +886,7 @@ class Notes:
             return []
 
         notes_tree_obj = NotesTree(notes_tree, self._object_store)
-        result = []
+        result: list[tuple[ObjectID, bytes]] = []
         for object_sha, note_sha in notes_tree_obj.list_notes():
             note_obj = self._object_store[note_sha]
             if isinstance(note_obj, Blob):
