@@ -21,7 +21,19 @@
 
 """Sparse checkout pattern handling."""
 
+__all__ = [
+    "BlobNotFoundError",
+    "SparseCheckoutConflictError",
+    "apply_included_paths",
+    "compute_included_paths_cone",
+    "compute_included_paths_full",
+    "determine_included_paths",
+    "match_gitignore_patterns",
+    "parse_sparse_patterns",
+]
+
 import os
+from collections.abc import Sequence, Set
 from fnmatch import fnmatch
 
 from .file import ensure_dir_exists
@@ -38,7 +50,9 @@ class BlobNotFoundError(Exception):
     """Raised when a requested blob is not found in the repository's object store."""
 
 
-def determine_included_paths(index: Index, lines: list[str], cone: bool) -> set[str]:
+def determine_included_paths(
+    index: Index, lines: Sequence[str], cone: bool
+) -> set[str]:
     """Determine which paths in the index should be included based on either a full-pattern match or a cone-mode approach.
 
     Args:
@@ -55,7 +69,7 @@ def determine_included_paths(index: Index, lines: list[str], cone: bool) -> set[
         return compute_included_paths_full(index, lines)
 
 
-def compute_included_paths_full(index: Index, lines: list[str]) -> set[str]:
+def compute_included_paths_full(index: Index, lines: Sequence[str]) -> set[str]:
     """Use .gitignore-style parsing and matching to determine included paths.
 
     Each file path in the index is tested against the parsed sparse patterns.
@@ -78,7 +92,7 @@ def compute_included_paths_full(index: Index, lines: list[str]) -> set[str]:
     return included
 
 
-def compute_included_paths_cone(index: Index, lines: list[str]) -> set[str]:
+def compute_included_paths_cone(index: Index, lines: Sequence[str]) -> set[str]:
     """Implement a simplified 'cone' approach for sparse-checkout.
 
     By default, this can include top-level files, exclude all subdirectories,
@@ -135,7 +149,7 @@ def compute_included_paths_cone(index: Index, lines: list[str]) -> set[str]:
 
 
 def apply_included_paths(
-    repo: Repo, included_paths: set[str], force: bool = False
+    repo: Repo, included_paths: Set[str], force: bool = False
 ) -> None:
     """Apply the sparse-checkout inclusion set to the index and working tree.
 
@@ -164,12 +178,8 @@ def apply_included_paths(
             blob_obj = repo.object_store[index_entry.sha]
         except KeyError:
             return True
-        # Create a temporary blob for normalization
-        temp_blob = Blob()
-        temp_blob.data = disk_data
-        norm_blob = normalizer.checkin_normalize(
-            temp_blob, os.path.relpath(full_path, repo.path).encode()
-        )
+        disk_blob = Blob.from_string(disk_data)
+        norm_blob = normalizer.checkin_normalize(disk_blob, full_path.encode("utf-8"))
         norm_data = norm_blob.data
         if not isinstance(blob_obj, Blob):
             return True
@@ -230,7 +240,7 @@ def apply_included_paths(
                         f.write(blob.data)
 
 
-def parse_sparse_patterns(lines: list[str]) -> list[tuple[str, bool, bool, bool]]:
+def parse_sparse_patterns(lines: Sequence[str]) -> list[tuple[str, bool, bool, bool]]:
     """Parse pattern lines from a sparse-checkout file (.git/info/sparse-checkout).
 
     This simplified parser:
@@ -281,7 +291,7 @@ def parse_sparse_patterns(lines: list[str]) -> list[tuple[str, bool, bool, bool]
 
 def match_gitignore_patterns(
     path_str: str,
-    parsed_patterns: list[tuple[str, bool, bool, bool]],
+    parsed_patterns: Sequence[tuple[str, bool, bool, bool]],
     path_is_dir: bool = False,
 ) -> bool:
     """Check whether a path is included based on .gitignore-style patterns.
