@@ -34,12 +34,15 @@ def test_suite() -> unittest.TestSuite:
         "dumb",
         "index",
         "lfs",
+        "midx",
         "pack",
         "patch",
         "porcelain",
         "reftable",
         "repository",
         "server",
+        "sha256",
+        "sha256_packs",
         "utils",
         "web",
     ]
